﻿Public Class Person
    Implements IDisposable

    Private mName As String
    Private mBirthdate As Date
    Private mTotalDistance As Integer
    Private mPhones As New Hashtable
    Private Shared mCounter As Integer

    Public Event Walked(ByVal distance As Integer)

    Public Sub New()
        Phone("home") = "(212) 555 1234"
        Phone("mobile") = "(888) 555 4321"
        mCounter += 1
    End Sub

    Public Sub New(ByVal name As String, ByVal birthdate As Date)
        mName = name
        mBirthdate = birthdate
        Phone("home") = "(212) 555 1234"
        Phone("mobile") = "(888) 555 4321"
        mCounter += 1
    End Sub

    Default Public Property Phone(ByVal location As String) As String
        Get
            Return CStr(mPhones.Item(location))     'convert to string
        End Get
        Set(value As String)
            If mPhones.ContainsKey(location) Then   'if location already exists
                mPhones.Item(location) = value      'then overwrite it
            Else
                mPhones.Add(location, value)        'else insert new value
            End If
        End Set
    End Property

    Public Sub Walk(ByVal distance As Integer)
        mTotalDistance += distance
        RaiseEvent Walked(distance)
    End Sub

    Public Sub Walk()
        RaiseEvent Walked(0)
    End Sub

    Public Function Age() As Integer
        'Return DateDiff(DateInterval.Year, mBirthdate, Now)

        'better:
        Dim age1 = Date.Now.Year - mBirthdate.Year
        'this can be 1 to high if this year's birthdate is in the future
        If mBirthdate.AddYears(age1) > Date.Now Then 'this year's birthdate
            age1 -= 1                               'correct age
        End If
        Return age1
    End Function

    Public Property Name() As String
        Get
            Return mName
        End Get
        Set(value As String)
            If String.IsNullOrWhiteSpace(value) Then Throw New Exception("Invalid name.")
            mName = value
        End Set
    End Property

    Public Shared ReadOnly Property Count() As Integer
        Get
            Return mCounter
        End Get
    End Property

    Public Shared Function CompareAge(ByVal person1 As Person, ByVal person2 As Person) As Boolean
        Return person1.Age > person2.Age
    End Function

    Public Shared Function CompareAge(ByVal person1 As Object, ByVal person2 As Object) As Boolean
        Return CType(person1, Person).Age > CType(person2, Person).Age
    End Function

    Public Shared Function CompareName(ByVal person1 As Object, ByVal person2 As Object) As Boolean
        Return CType(person1, Person).Name > CType(person2, Person).Name
    End Function

#Region "IDisposable Support"
    Private disposedValue As Boolean ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: dispose managed state (managed objects).
            End If

            ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
            ' TODO: set large fields to null.
        End If
        Me.disposedValue = True
    End Sub

    ' TODO: override Finalize() only if Dispose(ByVal disposing As Boolean) above has code to free unmanaged resources.
    'Protected Overrides Sub Finalize()
    '    ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class
