﻿'funktioniert
Module Module1

    Sub Main()
        Dim mStudents As New List(Of Student)

        'console template
        Console.WindowHeight = 30
        Console.WindowWidth = 120

        Console.ForegroundColor = ConsoleColor.Magenta
        Dim title As String = "New Generic Delegates, V1.0, H. M.Feichtinger, Copyright (C) 2009"
        Console.Title = title
        Console.WriteLine(title & vbCrLf)
        Console.WriteLine("Started " & Date.Now.ToLongTimeString)
        Console.ForegroundColor = ConsoleColor.Gray

        'define unhandled exception handler for all threads in this appdomain
        AddHandler AppDomain.CurrentDomain.UnhandledException, AddressOf DomainUnhandledExceptionHandler

        Try
            Call LoadStudentsData(mStudents)         'load students' data into list

            'Action(of T)		=> 
            'invoke a delegate on all the items in a collection with a single call.
            'output each student's ToString() info
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Action(T): output each student's ToString() info")
            Console.ForegroundColor = ConsoleColor.Gray
            mStudents.ForEach(AddressOf Print)      'output each student's ToString() info

            'Func(of T)			=> 
            'Encapsulates a method that has one parameter and returns a value of the type specified by the TResult parameter.
            'output "Name, Firstname" for all students
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Func(T): display canonical name of each student")
            Console.ForegroundColor = ConsoleColor.Gray
            Dim convertMethod As Func(Of Student, String) = AddressOf CanonicalName ' Instantiate delegate to reference UppercaseString method
            For Each s In mStudents
                Console.WriteLine(convertMethod(s)) ' Use delegate instance to call CanonicalName method
            Next

            'Comparison(of T)	=> 
            'Represents the method that compares two objects of the same type.
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Comparison(T): Sort students by birthdate")
            Console.ForegroundColor = ConsoleColor.Gray
            mStudents.Sort(AddressOf BirthdateComparer)
            For Each s In mStudents
                Console.WriteLine(s.Name & vbTab & s.BirthDate)
            Next

            'Converter(of T)	=> 
            'Represents a method that converts an object from one type to another type.
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Converter(T): convert students to graduates")
            Console.ForegroundColor = ConsoleColor.Gray
            Dim graduates As List(Of Graduate)
            graduates = mStudents.ConvertAll(AddressOf MakeGraduate)
            For Each g In graduates
                Console.WriteLine("Graduated: " & g.Name)
            Next

            'Predicate(of T)	=> 
            'Represents the method that defines a set of criteria and determines whether the specified object 
            'meets those criteria.
            'check whether as student is of full age (older than 18 years)
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Predicate(T): find first student which is of full age")
            Console.ForegroundColor = ConsoleColor.Gray
            Dim firstStudentOfFullAge = mStudents.Find(AddressOf OfFullAge)
            If Not IsNothing(firstStudentOfFullAge) Then _
                 Console.WriteLine(firstStudentOfFullAge.ToString & ": Is of full age.")

        Catch ex As Exception
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine(vbCrLf & "Error in Main: " & ex.Message)
            Console.ForegroundColor = ConsoleColor.Gray
            Stop
        Finally

        End Try

        Console.ForegroundColor = ConsoleColor.Red
        Console.WriteLine(vbCrLf & "Hit Enter to exit.")
        Console.ForegroundColor = ConsoleColor.Gray
        Console.ReadLine()
    End Sub

#Region "unhandled exception handler"
    'unhandled exception handler catches all threads!
    Sub DomainUnhandledExceptionHandler(ByVal sender As Object, ByVal args As UnhandledExceptionEventArgs)
        Dim e As Exception = DirectCast(args.ExceptionObject, Exception)
        Console.WriteLine("Unhandled Domain Exeption: " + e.Message)
    End Sub
#End Region

#Region "Delegates(of T)"
    Private Sub Print(ByVal s As Student)
        Console.WriteLine(s.ToString)
    End Sub

    Private Function CanonicalName(ByVal s As Student) As String
        Return s.Name & ", " & s.FirstName
    End Function

    Private Function OfFullAge(ByVal s As Student) As Boolean
        If (Date.Now.Year - s.BirthDate.Year) > 18 Then Return True Else Return False
    End Function

    Public Function MakeGraduate(ByVal s As Student) As Graduate
        Return New Graduate(s.Gender, s.Name, s.FirstName, s.BirthDate, Date.Now, 1)    'graduated today with level 1
    End Function

    Public Function BirthdateComparer(ByVal s1 As Student, ByVal s2 As Student) As Integer 'It returns an integer value indicating if the first object is greater than, less than, or equal to the second object.
        If s1.BirthDate < s2.BirthDate Then
            Return -1
        ElseIf s1.BirthDate > s2.BirthDate Then
            Return 1
        Else
            Return 0
        End If
    End Function
#End Region

#Region "Helper"
    Private Sub LoadStudentsData(ByRef students As List(Of Student))
        Throw New NotImplementedException("Geben Sie in LoadStudentsData Testdaten ein!")
        'create students:
        'students.Add(New Student(Genders.Female, "Name", "Vorname", #4/9/1989#))
        'students.Add(New Student(Genders.Male, "Mustermann", "Max", #9/20/1988#))

    End Sub
#End Region
End Module
