Imports System.Text
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.IO
Imports System.Xml.Serialization

<Serializable()> _
Public Class School

    Private mDepartments As New List(Of Department)
    Public Property Departments() As List(Of Department)
        Get
            Return mDepartments
        End Get
        Set(ByVal value As List(Of Department))
            mDepartments = value
        End Set
    End Property


    Private Shared mCount As Integer = 0
    Private mID As Integer

    Public ReadOnly Property ID() As Integer
        Get
            Return mID
        End Get
    End Property


    Private mName As String
    Public Property Name() As String
        Get
            Return mName
        End Get
        Set(ByVal value As String)
            mName = value
        End Set
    End Property

    Public Sub New(ByVal name As String)
        mID = mCount
        mCount += 1
        mName = name
    End Sub

    Private Sub New()   'for XML serialization

    End Sub

    Public Overrides Function ToString() As String
        Dim sb As New StringBuilder
        'sb.Append("School object model by HMF" & vbCrLf)
        sb.Append("School ID: " & mID & ", Name: " & mName & ", Nr. of Departments: " & mDepartments.Count)
        For Each d As Department In mDepartments
            sb.Append(vbCrLf & vbTab & d.ToString)
        Next
        Return sb.ToString
    End Function

    Public Sub SortStudentsByGender()
        For Each d As Department In mDepartments
            d.SortStudentsByGender()
        Next
    End Sub

    Public Sub SortStudentsBySunSign()  'Sternzeichen
        For Each d As Department In mDepartments
            d.SortStudentsBySunSign()
        Next
    End Sub

    Public Sub SortStudentsByBirthdate()
        For Each d As Department In mDepartments
            d.SortStudentsByBirthdate()
        Next
    End Sub

#Region "Binary Serialization"
    Public Sub SerializeBinary(ByVal file As String)
        Dim form As New BinaryFormatter()
        Dim stream As FileStream
        stream = New FileStream(file, FileMode.Create, FileAccess.Write, FileShare.None)
        form.Serialize(stream, Me)
        stream.Close()
    End Sub

    Public Shared Function DeserializeBinary(ByVal file As String) As School
        Dim s As School
        Dim form As New BinaryFormatter()
        Dim stream As FileStream
        stream = New FileStream(file, FileMode.Open, FileAccess.Read, FileShare.Read)
        s = form.Deserialize(stream)
        stream.Close()
        Return s
    End Function
#End Region

#Region "XML Serialization"
    Public Sub SerializeXML(ByVal file As String)
        Dim serializer As New XmlSerializer(Me.GetType)
        Dim writer As New StreamWriter(file)
        serializer.Serialize(writer, Me)
        writer.Close()
    End Sub

    Public Shared Function DeserializeXML(ByVal file As String) As School
        Dim s As School
        Dim reader As New StreamReader(file)
        Dim serializer As New XmlSerializer(GetType(School))
        s = serializer.Deserialize(reader)
        reader.Close()
        Return s
    End Function
#End Region

#Region "Create test data htl-donaustadt"
    Public Shared Function CreateSchoolTestData() As School
        'create students for 5ADH
        Dim sc1 As New SchoolClass("5ADH", "D200")
        sc1.Students.Add(New Student(Genders.Female, "B", "P", #4/9/1989#))
        sc1.Students.Add(New Student(Genders.Male, "B", "D", #9/20/1988#))
        sc1.Students.Add(New Student(Genders.Male, "B", "C", #9/24/1988#))
        sc1.Students.Add(New Student(Genders.Male, "G", "S", #8/1/1987#))
        sc1.Students.Add(New Student(Genders.Male, "H", "F", #8/23/1987#))
        sc1.Students.Add(New Student(Genders.Male, "L", "M", #11/21/1989#))
        sc1.Students.Add(New Student(Genders.Male, "P", "M", #12/4/1989#))
        sc1.Students.Add(New Student(Genders.Male, "S", "A", #12/11/1989#))
        sc1.Students.Add(New Student(Genders.Male, "S", "E", #7/31/1990#))
        sc1.Students.Add(New Student(Genders.Male, "V", "S", #5/19/1988#))
        sc1.Students.Add(New Student(Genders.Female, "V", "M", #10/29/1989#))
        sc1.Students.Add(New Student(Genders.Male, "V", "M", #5/26/1990#))
        sc1.Students.Add(New Student(Genders.Male, "W", "P", #7/21/1989#))
        sc1.Students.Add(New Student(Genders.Male, "W", "T", #11/30/1987#))

        'create students for 5BDH
        Dim sc2 As New SchoolClass("5BDH", "D200")
        sc2.Students.Add(New Student(Genders.Male, "B", "K", #5/3/1990#))
        sc2.Students.Add(New Student(Genders.Male, "B", "M", #7/7/1990#))
        sc2.Students.Add(New Student(Genders.Male, "B", "A", #8/8/1988#))
        sc2.Students.Add(New Student(Genders.Male, "C", "A", #3/6/1990#))
        sc2.Students.Add(New Student(Genders.Female, "D", "", #10/18/1988#))
        sc2.Students.Add(New Student(Genders.Male, "E", "M", #12/5/1989#))
        sc2.Students.Add(New Student(Genders.Male, "F", "T", #1/28/1990#))
        sc2.Students.Add(New Student(Genders.Male, "F", "R", #8/12/1989#))
        sc2.Students.Add(New Student(Genders.Male, "K", "C", #8/31/1990#))
        sc2.Students.Add(New Student(Genders.Female, "L", "V", #3/9/1990#))
        sc2.Students.Add(New Student(Genders.Male, "M", "R", #11/16/1988#))
        sc2.Students.Add(New Student(Genders.Male, "N", "L", #4/4/1990#))
        sc2.Students.Add(New Student(Genders.Male, "P", "D", #8/25/1986#))
        sc2.Students.Add(New Student(Genders.Female, "S", "C", #2/17/1990#))
        sc2.Students.Add(New Student(Genders.Female, "S", "N", #2/14/1988#))
        sc2.Students.Add(New Student(Genders.Male, "S", "A", #3/19/1990#))
        sc2.Students.Add(New Student(Genders.Male, "W", "D", #10/28/1989#))

        'create students for 5EDH
        Dim sc3 As New SchoolClass("5EDH", "D200")
        sc3.Students.Add(New Student(Genders.Male, "B", "R", #9/18/1989#))
        sc3.Students.Add(New Student(Genders.Male, "B", "B", #8/24/1990#))
        sc3.Students.Add(New Student(Genders.Male, "D", "A", #7/4/1990#))
        sc3.Students.Add(New Student(Genders.Male, "D", "A", #8/29/1989#))
        sc3.Students.Add(New Student(Genders.Male, "G", "S", #10/6/1989#))
        sc3.Students.Add(New Student(Genders.Male, "G", "D", #6/13/1990#))
        sc3.Students.Add(New Student(Genders.Male, "G", "M", #9/6/1989#))
        sc3.Students.Add(New Student(Genders.Male, "H", "M", #3/4/1990#))
        sc3.Students.Add(New Student(Genders.Male, "H", "P", #2/22/1988#))
        sc3.Students.Add(New Student(Genders.Male, "K", "P", #1/19/1990#))
        sc3.Students.Add(New Student(Genders.Male, "L", "C", #11/26/1989#))
        sc3.Students.Add(New Student(Genders.Male, "S", "D", #5/4/1990#))
        sc3.Students.Add(New Student(Genders.Male, "S", "A", #9/5/1989#))
        sc3.Students.Add(New Student(Genders.Male, "S", "P", #8/28/1989#))
        sc3.Students.Add(New Student(Genders.Male, "T", "S", #7/9/1988#))
        sc3.Students.Add(New Student(Genders.Male, "T", "P", #7/1/1989#))
        sc3.Students.Add(New Student(Genders.Male, "V", "J", #4/6/1989#))

        'create students for 4ADH
        Dim sc4 As New SchoolClass("4ADH", "D200")
        sc4.Students.Add(New Student(Genders.Male, "A", "J", #3/24/1990#))
        sc4.Students.Add(New Student(Genders.Female, "A", "L", #11/25/1989#))
        sc4.Students.Add(New Student(Genders.Male, "C", "B", #10/9/1989#))
        sc4.Students.Add(New Student(Genders.Female, "D", "K", #4/28/1991#))
        sc4.Students.Add(New Student(Genders.Male, "E", "R", #11/2/1990#))
        sc4.Students.Add(New Student(Genders.Female, "H", "S", #1/11/1991#))
        sc4.Students.Add(New Student(Genders.Male, "H", "M", #1/15/1986#))
        sc4.Students.Add(New Student(Genders.Male, "J", "R", #12/13/1990#))
        sc4.Students.Add(New Student(Genders.Male, "K", "P", #4/1/1991#))
        sc4.Students.Add(New Student(Genders.Male, "K", "Y", #11/2/1988#))
        sc4.Students.Add(New Student(Genders.Male, "K", "C", #5/5/1991#))
        sc4.Students.Add(New Student(Genders.Female, "L", "P", #10/17/1989#))
        sc4.Students.Add(New Student(Genders.Male, "M", "R", #9/4/1991#))
        sc4.Students.Add(New Student(Genders.Male, "N", "P", #3/29/1991#))
        sc4.Students.Add(New Student(Genders.Male, "O", "P", #6/10/1991#))
        sc4.Students.Add(New Student(Genders.Male, "P", "M", #5/4/1991#))
        sc4.Students.Add(New Student(Genders.Male, "S", "S", #5/28/1991#))
        sc4.Students.Add(New Student(Genders.Male, "S", "C", #6/8/1991#))
        sc4.Students.Add(New Student(Genders.Male, "S", "Z", #6/11/1990#))
        sc4.Students.Add(New Student(Genders.Male, "S", "A", #4/26/1991#))
        sc4.Students.Add(New Student(Genders.Male, "S", "C", #4/3/1991#))
        sc4.Students.Add(New Student(Genders.Female, "S", "H", #7/4/1990#))
        sc4.Students.Add(New Student(Genders.Male, "T", "M", #10/1/1999#))
        sc4.Students.Add(New Student(Genders.Male, "V", "C", #6/8/1991#))

        'create students for 4BDH
        Dim sc5 As New SchoolClass("4BDH", "D200")

        'create students for 4EDH
        Dim sc6 As New SchoolClass("4EDH", "D200")

        'create department
        Dim d1 As New Department("EDV-O")
        d1.SchoolClasses.Add(sc1)
        d1.SchoolClasses.Add(sc2)
        d1.SchoolClasses.Add(sc3)
        d1.SchoolClasses.Add(sc4)
        d1.SchoolClasses.Add(sc5)
        d1.SchoolClasses.Add(sc6)

        Dim d2 As New Department("Elektronik")
        Dim d3 As New Department("Abendschule")

        'create school
        Dim s As New School("htl-donaustadt")
        s.Departments.Add(d1)
        s.Departments.Add(d2)
        s.Departments.Add(d3)

        Return s
    End Function
#End Region

End Class
