'Composable Stream, der Read() implementiert.
'funktioniert mit single byte character, US-ASCII

'bung: vervollstndigen Sie den Stream (Seek, Write, Flush, ...)

Imports System.IO
Imports System.Text

Public Enum Conversions
    ToUpperCase
    ToLowerCase
    None
End Enum

''' <summary>
''' CaseConverterStream converts a stream of text between uppercase and lower case. Conversion can happen
''' on read and write operations. This is a composable stream.
''' </summary>
''' <remarks></remarks>
Public Class CaseConverterStream
    Inherits Stream

    Private mStream As Stream   'stream were to read data from 
    Private mConversion As Conversions  'type of conversion: ToUpperCase, ToLowerCase

    ''' <summary>
    ''' Converter Stream between upper case characters and lower case characters
    ''' </summary>
    ''' <param name="stream">input stream where data is read from</param>
    ''' <param name="conversion">Enum ToUpper/ToLower</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal stream As Stream, ByVal conversion As Conversions)
        If stream Is Nothing Then Throw New ArgumentNullException("Stream must not be nothing.")
        mStream = stream
        mConversion = conversion
    End Sub

    Public Overrides ReadOnly Property CanRead() As Boolean
        Get
            Return mStream.CanRead
        End Get
    End Property

    Public Overrides ReadOnly Property CanSeek() As Boolean
        Get
            Return mStream.CanSeek
        End Get
    End Property

    Public Overrides ReadOnly Property CanWrite() As Boolean
        Get
            Return False
        End Get
    End Property

    Public Overrides Sub Flush()

    End Sub

    Public Overrides ReadOnly Property Length() As Long
        Get
            Return mStream.Length
        End Get
    End Property

    Public Overrides Property Position() As Long
        Get
            Return mStream.Position
        End Get
        Set(ByVal value As Long)
            mStream.Position = value
        End Set
    End Property

    ''' <summary>
    ''' Read from the input stream specified in the constructor, convert and return the data.
    ''' </summary>
    ''' <param name="buffer">output buffer containing converted characters</param>
    ''' <param name="offset">position of first character in buffer</param>
    ''' <param name="count">number of characters written, starting at offset</param>
    ''' <returns>Nr of processed characters</returns>
    ''' <remarks></remarks>
    Public Overrides Function Read(ByVal buffer() As Byte, ByVal offset As Integer, ByVal count As Integer) As Integer

        Dim readCount As Integer
        Dim localBuffer(count - 1) As Byte  'create an interim buffer the same size as our output buffer

        readCount = mStream.Read(localBuffer, 0, count)  'read from input stream into local buffer
        For i As Integer = 0 To readCount - 1
            'debug: output the character at position i in buffer

            'convert the character
            If mConversion = Conversions.ToLowerCase Then
                Dim c As Char = Chr(localBuffer(i))     'convert to character
                buffer(i) = Asc(Char.ToLower(c))        'convert back to byte

            ElseIf mConversion = Conversions.ToUpperCase Then
                Dim c As Char = Chr(localBuffer(i))
                buffer(i) = Asc(Char.ToUpper(c))
            Else            'do nothing (this may be useful for tests
                'return the buffer to the caller; copy to output buffer without conversion
                buffer(i + offset) = localBuffer(i)
            End If
        Next
        Return readCount
    End Function

    Public Overrides Function Seek(ByVal offset As Long, ByVal origin As System.IO.SeekOrigin) As Long
        Throw New NotImplementedException()
    End Function

    Public Overrides Sub SetLength(ByVal value As Long)
        Throw New NotImplementedException()
    End Sub

    ''' <summary>
    ''' Convert the data and write it to the output stream specified in the constructor.
    ''' </summary>
    ''' <param name="buffer"></param>
    ''' <param name="offset"></param>
    ''' <param name="count"></param>
    ''' <remarks></remarks>
    Public Overrides Sub Write(ByVal buffer() As Byte, ByVal offset As Integer, ByVal count As Integer)
        Throw New NotImplementedException()
    End Sub
End Class
