﻿Public Enum NameType
    Formal
    Nick
End Enum

Public Class Student
    Inherits Person
    Implements IPrintable


#Region "Variables"
    Private mEntryDate As Date
    Private mPocketMoney As Double
    Private mNames As New Dictionary(Of NameType, String)
    Private mLabels() As String = {"ID", "PocketMoney", "EntryDate"}
#End Region

#Region "Constructors"
    Public Sub New()
        Debug.WriteLine("Student constructor")
    End Sub

    Public Sub New(name As String, birthDate As Date)
        MyBase.New(name, birthDate)
    End Sub
#End Region

#Region "Properties"
    Public Overloads Property Name(ByVal type As NameType) As String
        Get
            Return mNames(type)
        End Get
        Set(value As String)
            mNames.Add(type, value)
        End Set
    End Property

    Public Property EntryDate As Date
        Get
            Return mEntryDate
        End Get
        Set(value As Date)
            mEntryDate = value
        End Set
    End Property

    Public Property PocketMoney As Double
        Get
            Return mPocketMoney
        End Get
        Set(value As Double)
            mPocketMoney = value
        End Set
    End Property

    Public Overrides Property BirthDate As Date
        Get
            Return MyBase.BirthDate
        End Get
        Set(value As Date)
            If DateDiff(DateInterval.Year, value, Now) >= 14 Then
                MyBase.BirthDate = value
            Else
                Throw New ArgumentException("A student must be elder than 14 years.")
            End If
        End Set
    End Property

    Public Shadows ReadOnly Property Age As Integer
        Get
            Return CInt(DateDiff(DateInterval.Year, BirthDate, Now))
        End Get
    End Property
#End Region

    Private ReadOnly Property Count As Integer Implements IPrintable.Count
        Get
            Return mLabels.Count
        End Get
    End Property

    Private Function Label(index As Integer) As String Implements IPrintable.Label
        Return mLabels(index)
    End Function

    Private Function Value(index As Integer) As String Implements IPrintable.Value
        Select Case index
            Case 0
                Return Name
            Case 1
                Return mPocketMoney
            Case 2
                Return Format(EntryDate, "Short date")
            Case Else
                Throw New Exception("Invalid index. must be 0..2")
        End Select
    End Function
End Class

